/*  Ishape2d.pro
Material description (for getdp) of Ishape2D test for ECE
For details, see comments in the Ishape2d_data.pro file
The Ishape2d geometry and meshing parameters are in the Ishape2d.geo file
This file contains the setings that depend on the particular problem and
it includes 3 general files that refer to:
- computation of Jacobians
- numerical integration settings
- the weak FW formulations (where classic and ECE BC are used), and EC just to test the results for low freq.
*/

Include "Ishape2d_data.pro"

Group{
 /* Definition of the interface between the mesh file and GetDP */
 /* Elementary groups */

 MaterialX = Region[100]; /* MaterialX - homogenous domain */
 Ground = Region[120];    /* Boundary - various parts */
 Terminal = Region[121];
 RightBoundary = Region[131];
 LeftBoundary = Region[132];
 
 Vol_FW = Region[{MaterialX}]; // domain without the boundary
 Sur_Terminals_FWece = Region[{Ground, Terminal}]; // all terminals
 Sur_FW = Region[{Sur_Terminals_FWece, RightBoundary, LeftBoundary}]; //all boundary
 //Dom_FW = Region[ {Vol_FW, RightBoundary, LeftBoundary} ];
 Dom_FW = Region[ {Vol_FW, Sur_FW} ];
 
 If (Flag_AnalysisType==0)       // classic BC, Et everywhere
   Sur_H_FWclassic  = Region[{}]; 	   // surface where Ht is imposed, here the empty set
   Sur_E_FWclassic = Region[{Sur_FW}]; // surface where Et is imposed
 ElseIf (Flag_AnalysisType==1)   // classic BC, Et on top and bottom, Ht on left and right
   Sur_H_FWclassic  = Region[{RightBoundary,LeftBoundary}]; // surface where Ht is imposed
   Sur_E_FWclassic  = Region[{Terminal,Ground}]; // surface where Et is imposed
 EndIf
 
 If ((Flag_AnalysisType==4)||(Flag_AnalysisType==5))    // EC ece BC
	 Vol_EC = Region[{Vol_FW}]; // domain without the boundary
	 Sur_Terminals_ECece = Region[{Sur_Terminals_FWece}]; // all terminals
	 Sur_EC = Region[{Sur_FW}]; //all boundary
	 Dom_EC = Region[ {Dom_FW} ];
 EndIf
  
}

 Function{
   /* Material properties are defined piecewise, for elementary groups */
   /* the values in the right hand side are defined in Ishape2d_data.pro */

   nu[#{MaterialX,Sur_FW}] = nu;  // magnetic reluctivity [m/H]
   epsilon[#{MaterialX,Sur_FW}] = eps;       // permittivity [F/m]
   sigma[#{MaterialX,Sur_FW}]  = sigma;      // conductivity [S/m]

   // FW in E, classic BC, Et everywhere, time harmonic
   BC_Fct_e[Terminal] = EtBottomRMS*Complex[Cos[EtBottomPhase],Sin[EtBottomPhase]] * Vector[1, 0, 0];
   BC_Fct_e[Ground]   = EtTopRMS   *Complex[Cos[EtTopPhase],   Sin[EtTopPhase]] * Vector[1, 0, 0];
   If(Flag_AnalysisType==0)
     BC_Fct_e[LeftBoundary]  = EtLeftRMS * Complex[Cos[EtLeftPhase], Sin[EtLeftPhase]] * Vector[0, 1, 0];
     BC_Fct_e[RightBoundary] = EtRightRMS* Complex[Cos[EtRightPhase],Sin[EtRightPhase]] * Vector[0, 1, 0];
   EndIf

   // FW in E, classic BC, Et on top and bottom, Ht on left and right
   BC_Fct_h[LeftBoundary]  = HtLeftRMS*Complex[Cos[HtLeftPhase],Sin[HtLeftPhase]] * Vector[0, 0, 1];
   BC_Fct_h[RightBoundary] = HtRightRMS*Complex[Cos[HtRightPhase],Sin[HtRightPhase]] * Vector[0, 0, -1];

   // FW in E, ece BC, voltage excitation on bottom, ground on top
   // When imposing voltage or current via a constraint, the region appears there
   // If you have a complex, the value has to be a function => use square brackets []
   VBottom[] = (Flag_AnalysisType==4)? VBottom : -VBottomRMS*Complex[Cos[VBottomPhase],Sin[VBottomPhase]];
   // FW in E, ece BC, current excitation on bottom (entering the domain), ground on top, time harmonic
   IBottom[] = (Flag_AnalysisType==5)? IBottom : -IBottomRMS*Complex[Cos[IBottomPhase],Sin[IBottomPhase]];
 }

 Constraint{

   { Name ElectricVoltages;
     Case {
       If ((Flag_AnalysisType==0)||(Flag_AnalysisType==1))     // classic BC
         { Region Sur_E_FWclassic ; Type AssignFromResolution ; NameOfResolution VoltageSource_e_BC ; }
       EndIf
     }
   }

   // ece BC or electrokinetics
   { Name SetTerminalPotential; Type Assign;  // voltage excited terminals
     Case {
       { Region Ground; Value 0.; }
       If((Flag_AnalysisType==2)||(Flag_AnalysisType==4))
         { Region Terminal; Value VBottom[]; }
       EndIf
     }
   }
   { Name SetTerminalCurrent; Type Assign;   // current excited terminals
     Case {
       If((Flag_AnalysisType==3)|(Flag_AnalysisType==5))
         { Region Terminal; Value IBottom[]/h2Ddepth; }  // here the depth is needed
       EndIf
     }
   }

}

 /* The formulation and its tools */

 If ((Flag_AnalysisType==0)|(Flag_AnalysisType==1))      // FW in E, clasic BC
   Include "../problemIndependent_proFiles/FullWave_E_classicBC.pro"
 ElseIf ((Flag_AnalysisType==2)|(Flag_AnalysisType==3))  // FW in E, ECE
   Include "../problemIndependent_proFiles/FullWave_E_ece_SISO.pro"
 Else                                                    // electrokinetics
    Include "../problemIndependent_proFiles/Electrokinetics_V_ece.pro"
 EndIf


 If (Flag_AnalysisType==0)
	Dir = "res/FWclassicBC_EtEverywhere/";
 ElseIf (Flag_AnalysisType==1)
	Dir = "res/FWclassicBC_EtandHt/";
 ElseIf (Flag_AnalysisType==2)
	Dir = "res/FWeceBC_voltExc/";
 ElseIf (Flag_AnalysisType==3)
	Dir = "res/FWeceBC_crtExc/";
 ElseIf (Flag_AnalysisType==4)
	Dir = "res/EC_voltExc/";
 Else
	Dir = "res/EC_crtExc/";
 EndIf
 /* Output results */
Macro Change_post_options
Echo[Str[ "nv = PostProcessing.NbViews;",
    "For v In {0:nv-1}",
    "View[v].NbIso = 25;",
    "View[v].RangeType = 3;" ,// per timestep
    "View[v].ShowTime = 3;",
    "View[v].IntervalsType = 3;",
    "EndFor"
  ], File "post.opt"];
Return



 PostOperation {
 If ((Flag_AnalysisType==0)||(Flag_AnalysisType==1))  // clasic BC
  { Name Maps; NameOfPostProcessing FW_E_classic ;
     Operation {
       Print [ E, OnElementsOf Vol_FW, File StrCat[Dir,"map_E.pos" ]];
       // Print [ E, OnElementsOf Sur_E_FWclassic, File StrCat[Dir,"map_Et.pos" ]];

       //Print [ B, OnElementsOf Vol_FW, File StrCat[Dir,"map_B.pos" ]];
       Print [ H, OnElementsOf Vol_FW, File StrCat[Dir,"map_H.pos" ]];
       //Print [ H, OnElementsOf Sur_H_FWclassic, File StrCat[Dir,"map_Ht.pos" ]];
       Call Change_post_options;  // changes options of gmsh
     }
  }
  ElseIf ((Flag_AnalysisType==2)||(Flag_AnalysisType==3))  //  ECE
  { Name Maps; NameOfPostProcessing FW_E_ece ;
     Operation {
       Print [ gradV, OnElementsOf ElementsOf[Vol_FW,OnOneSideOf Sur_FW], File StrCat[Dir,"map_gradV.pos" ]];
       
       Print [ E, OnElementsOf Vol_FW, File StrCat[Dir,"map_E.pos" ]];
       Print [ J, OnElementsOf Vol_FW, File StrCat[Dir,"map_J.pos" ]];
       Print [ rmsJ, OnElementsOf Vol_FW, File StrCat[Dir,"map_absJ.pos" ]];

       // Print [ B, OnElementsOf Vol_FW, File StrCat[Dir,"map_B.pos" ]];
       Print [ H, OnElementsOf Vol_FW, File StrCat[Dir,"map_H.pos" ]];
       //Print [ Dummy, OnElementsOf Vol_FW, File StrCat[Dir,"map_dummy.pos" ]];
	   Print [ Vterminals, OnElementsOf ElementsOf[Sur_FW], File StrCat[Dir,"map_Vterminals.pos" ]];
	   Print [ VnotTerminals, OnElementsOf Vol_FW, File StrCat[Dir,"map_VnotTerminals.pos" ]];

       Call Change_post_options;
     }
  }
  Else // Electrokinetic
  { Name Maps; NameOfPostProcessing EC_v_terminals ;
     Operation {
       Print [ E, OnElementsOf Vol_EC, File StrCat[Dir,"map_E.pos" ]];
       Print [ v, OnElementsOf Vol_EC, File StrCat[Dir,"map_V.pos" ]]; 

       Call Change_post_options;
     }
  }
  EndIf
}



