/* Ishape2d_data.pro
=========================== Ishape2D - test problem
A rectangular domain in the plane xOy, covering [-a,a] X [0,l]
(width = 2a, height = l);
A depth h (dimension along Oz) is needed for postrocessing or for the current
excitation case.
The domain is filled with homogeneous and linear material, with epsilon, mu, sigma.
There is no no field source inside the domein (the "device" is passive).
The field sources are only boundary conditions (BC) which can be eiter (exclusive or)
1) classical BC - Et and Ht given in any point of the boundary
2) ECE BC - surface satisfies ECE BC, meaning that only the position of terminals has to
be given, their excitation type (voltage xor current) and their excitation signals
(frequency domain simulations, i.e. root means square values and initial phases have to
be given for the input signals)

This is a test problem - used for the first validation of the ECE implementation in onelab.
As an ECE BC, it is a device with 2 terminals, out of which one is ground.
Thus, as a system this is a single input single output (SISO) system.

===========================
Names of input data - to be given by the user
Geometry:
a = half of the dimension on the Ox axis [m]
l = dimension along the Oy axis [m]
h = dimension along the Oz axis (useful for postprocessing and for current excited case) [m]
==
Material:
epsr = relative permittivity
mur = relative permeability
sigma = conductivity [S/m
==
Type of BC:
typeBC = 0,1,2,3
0 = classical Et everywhere (default values correspond to voltage excitation, 1 V on the bottom, ground (0 V) on top
1 = classical Et and Ht (default values correspond to current excitation, 1 A injected into the bottom term, top is ground (0 V)
2 = ECE, voltage excitation (1 V bottom, ground on top)
3 = ECE, current excitation (1 A injected into the bottom terminal, top terminal is grounded)
4 = EC (electrokinetics), Dirichlet boundary made of two terminals, the rest is zero Neumann. One terminal is ground, the other is voltage excited
5 = EC as above, but current excited terminal
==
Excitation signals
* If typeBC=0 => FW formulation in E, Et values are imposed everywhere (here on each side of the boundary a constant Et is assumed
default values: Et=0 on bottom and top, Et = 1/l, oriented in the positive Oy direction on left and right
* If typeBC = 1 => FW formulation in E, Et values are imposed on bottom and top (Et = 0), Ht values are imposed on left and right
Ht = 1/(2h) oriented in the pozitive Oz direction on the left part and Ht = 1/(2h) oriented opposite to Oz (i.e. negative Oz) on the right part
* If typeBC = 2 => FW formulation in E, ECE, 1 V on the bottom;  0 V  on the top
* If typeBC = 3 => FW formulation in E, ECE, 1 A on the bottom (entering the domain); 0V on the top
* If typeBC = 4 => EC formulation in V, terminals and the rest zero Neumann, 1 V on the bottom;  0 V  on the top
* If typeBC = 5 => EC formulation in V, terminals and the rest zero Neumann, 1 A on the bottom;  0 V  on the top
=========================== */

colorro = "LightGrey";

/* new menu for the interface - holds geometrical data */
cGUI= "{Ishape2D-ProblemData/";
mGeo = StrCat[cGUI,"Geometrical parameters/0"];
colorMGeo = "Ivory";
close_menu = 1;
DefineConstant[
  a = {2.5e-6, Name StrCat[mGeo, "0a=0.5*Dimension along Ox"], Units "m", Highlight Str[colorMGeo], Closed close_menu},
  l = {10e-6, Name StrCat[mGeo, "1l=Dimension along Oy"], Units "m", Highlight Str[colorMGeo]},
  h = {10e-6, Name StrCat[mGeo, "2h=Dimension along Oz"],Units "m", Highlight Str[colorMGeo]}
];

/* new menu for the interface - holds material data */
mMat = StrCat[cGUI,"Material parameters/0"];
colorMMat = "AliceBlue";
DefineConstant[
  epsr = {1, Name StrCat[mMat, "0Relative permittivity"], Units "-", Highlight Str[colorMMat], Closed close_menu},
  mur = {1, Name StrCat[mMat, "1Relative permeability"], Units "-", Highlight Str[colorMMat]},
  sigma = {6.6e7, Name StrCat[mMat, "2Conductivity"], Units "S/m", Highlight Str[colorMMat]}
];

eps0 = 8.854187818e-12; // F/m - absolute permitivity of vacuum
mu0  = 4.e-7 * Pi;      // H/m - absolute permeability of vacuum
nu0  = 1/mu0;           // m/H - absolute reluctivity of vacuum

eps = eps0*epsr;        // F/m - absolute permitivity
mu = mu0*mur;           // F/m - absolute permeability
nu = 1/mu;              // m/H - absolute reluctivity

/* new menu for the interface - type of BC */
mTypeBC = StrCat[cGUI,"Type of BC/0"];
colorMTypeBC = "Red";
DefineConstant[
  Flag_AnalysisType = { 0,
    Choices{
      0="classical: Et everywhere",
      1="classical: Et on top and bottom, Ht on left and right",
      2="ECE: voltage excitation on bottom, ground on top",
      3="ECE: current excitation on bottom, ground on top",
      4 = "Electrokinetics - voltage excited (non terminals are zero Neumann) just for test",
      5 = "Electrokinetics - current excited (non terminals are zero Neumann) just for test"
    },
    Name Str[mTypeBC], Highlight Str[colorMTypeBC], Closed !close_menu,
    Help Str["Classical Et everywhere can be made equivalent to ECE voltage excitation",
      "Classical Et and Ht can be made equivalent to ECE current excitation",
      "This equivalences are particular for this test problem only."],
    ServerAction Str["Reset", "GetDP/1ResolutionChoices"]}
];

/* new menu for the interface - values of excitations - significance depends on the type of BC */
mValuesBC = StrCat[cGUI,"Values of BC (frequency analysis)/0"];
colorMValuesBC = "Ivory";

fmin = 1e7;   // Hz
//fmed = 3e9;   // Hz
//fmax = 100e9; // Hz

DefineConstant[
  Freq  = {fmin, Name StrCat[mValuesBC, "0Working Frequency"], Units "Hz", Highlight Str[colorMValuesBC], Closed !close_menu }
];
// You may also use UndefineConstant[]

DefineConstant[
  // Flag_AnalysisType
  // ==0 classic BC, Et everywhere
  // ==1 classic BC, Et on top and bottom, Ht on left and right
  // ==2 ECE, voltage excitation
  // ==3 ECE, current excitation
  // ==4 Elektrokinetics, voltage excitation
  // ==5 Elektrokinetics, current excitation
  EtBottomRMS  = {0, Name StrCat[mValuesBC, "1Bottom/1Et on bottom, Ox orientation (rms)"], Units "V/m", Highlight Str[colorMValuesBC],
    Visible (Flag_AnalysisType<2)}
  EtBottomPhase  = {0, Name StrCat[mValuesBC, "1Bottom/1Et on bottom, phase"], Units "rad", Highlight Str[colorMValuesBC],
    Visible (Flag_AnalysisType<2)}
  //
  EtTopRMS  = {0, Name StrCat[mValuesBC, "2Top/1Et on top, Ox orientation (rms)"], Units "V/m", Highlight Str[colorMValuesBC],
    Visible (Flag_AnalysisType<2)}
  EtTopPhase  = {0, Name StrCat[mValuesBC, "2Top/1Et on top, phase"], Units "rad", Highlight Str[colorMValuesBC],
    Visible (Flag_AnalysisType<2)}
  //
  EtLeftRMS  = {1/l, Name StrCat[mValuesBC, "3Left/1Et on left, Oy orientation (rms)"], Units "V/m", Highlight Str[colorMValuesBC],
    Visible (Flag_AnalysisType==0) }
  EtLeftPhase  = {0, Name StrCat[mValuesBC, "3Left/1Et on left, phase"], Units "rad", Highlight Str[colorMValuesBC],
    Visible (Flag_AnalysisType==0)}
  //
  EtRightRMS  = {1/l, Name StrCat[mValuesBC, "4Right/1Et on right, Oy orientation (rms)"], Units "V/m", Highlight Str[colorMValuesBC],
    Visible (Flag_AnalysisType==0)}
  EtRightPhase  = {0, Name StrCat[mValuesBC, "4Right/1Et on right, phase"], Units "rad", Highlight Str[colorMValuesBC],
    Visible (Flag_AnalysisType==0)}
  //
  HtLeftRMS  = {1/(2*h), Name StrCat[mValuesBC, "3Left/1Ht on left, Oz orientation (rms)"], Units "A/m", Highlight Str[colorMValuesBC],
    Visible (Flag_AnalysisType==1) }
  HtLeftPhase  = {0, Name StrCat[mValuesBC, "3Left/1Ht on left, phase"], Units "rad", Highlight Str[colorMValuesBC],
    Visible (Flag_AnalysisType==1) }
  //
  HtRightRMS  = {1/(2*h), Name StrCat[mValuesBC, "4Right/1Ht on right, -Oz orientation (rms)"], Units "A/m", Highlight Str[colorMValuesBC],
    Visible (Flag_AnalysisType==1)}
  HtRightPhase  = {0, Name StrCat[mValuesBC, "4Right/1Ht on right, phase"], Units "rad", Highlight Str[colorMValuesBC],
    Visible (Flag_AnalysisType==1)}

  // ECE, voltage excitation
  VBottomRMS  = {1, Name StrCat[mValuesBC, "Bottom/1Potential on bottom (rms)"], Units "V", Highlight Str[colorMValuesBC],
    Visible (Flag_AnalysisType==2)}
  VBottomPhase  = {0, Name StrCat[mValuesBC, "Bottom/1Potential on bottom, phase"], Units "rad", Highlight Str[colorMValuesBC],
    Visible (Flag_AnalysisType==2)}
  //
  VtTopRMS  = {0, Name StrCat[mValuesBC, "Top/1Potential on top (rms)"], Units "V" , ReadOnly 1, Highlight Str[colorro],
    Visible (Flag_AnalysisType==2) || (Flag_AnalysisType==3)}
  VtTopPhase  = {0, Name StrCat[mValuesBC, "Top/1Potential on top, phase"], Units "rad",  ReadOnly 1, Highlight Str[colorro],
    Visible  (Flag_AnalysisType==2) || (Flag_AnalysisType==3)}

  // ECE, current excitation
  IBottomRMS  = {1, Name StrCat[mValuesBC, "Bottom/1Current on bottom (enters the domain) (rms)"], Units "A", Highlight Str[colorMValuesBC],
    Visible (Flag_AnalysisType==3) }
  IBottomPhase  = {0, Name StrCat[mValuesBC, "Bottom/1Current on bottom, phase"], Units "rad", Highlight Str[colorMValuesBC],
    Visible (Flag_AnalysisType==3)}
  //
  // Elektrokinetics, voltage excitation
  VBottom  = {1, Name StrCat[mValuesBC, "Bottom/1Potential on bottom"], Units "V", Highlight Str[colorMValuesBC],
    Visible  (Flag_AnalysisType==4) }
  //
  VtTop  = {0, Name StrCat[mValuesBC, "Top/1Potential on top"], Units "V" , ReadOnly 1, Highlight Str[colorro],
    Visible  (Flag_AnalysisType==4) || (Flag_AnalysisType==5)}
  // Elektrokinetics, current excitation
  IBottom  = {1, Name StrCat[mValuesBC, "Bottom/1Current on bottom (enters the domain)"], Units "A", Highlight Str[colorMValuesBC],
    Visible  (Flag_AnalysisType==5) }
];


DefineConstant[
  _use_transfinite = {0, Choices{0,1}, Name "{MeshSettings/0Transfinite mesh?", Highlight "Yellow", Closed !close_menu}
   s = {10, Name "{MeshSettings/1Mesh factor"}
];

modelDim = 2; // 
Flag_Axi = 0; // 1 for AXI - not tested for the moment, it makes sense only for modelDim = 2

If ((modelDim == 2)&&(Flag_Axi == 0))
	h2Ddepth = h;
ElseIf ((modelDim == 2)&&(Flag_Axi == 1))   // 2D AXI
	h2Ddepth = 2*Pi;
Else // 3D
    h2Ddepth = 1;
EndIf
	