/* Tshape2d_data.pro
=========================== Tshape2D - test problem
A rectangular domain in the plane xOy, covering [-a,a] X [0,l]
(width = 2a, height = l);
A depth h (dimension along Oz) is needed for postrocessing or for the current
excitation case.
The domain is filled with homogeneous and linear material, with epsilon, mu, sigma
covering a T shape part drawn as rotated with 90 degrees conter clockwise, i.e. |-

The | part is sticked to the left boundary and its width is b > 0
The - part is sticked to the right boundary, its width is d > 0 and it is placed at a distance c 
from the horizontal part.

There is no no field source inside the domein (the "device" is passive).
The field sources are only boundary conditions (BC) which are ECE.

The problem has 3 terminals (on top, bottom and right), one has to be grounded, the other two can be either
voltage excited or current excited.


This is a test problem - used to validate the ECE implementation in onelab for a 
multiple input multiple output (MIMO) system.

===========================
Names of input data - to be given by the user
Geometry:
a = half of the dimension on the Ox axis [m]
l = dimension along the Oy axis [m]
h = dimension along the Oz axis (useful for postprocessing and for current excited case) [m]
b = width of the | part
c = distance from the bottom line of the - part
d = widht of the - part
==
Material of the T part
epsr1 = relative permittivity
mur1 = relative permeability
sigma1 = conductivity [S/m]

Material of the rest 
epsr2 = relative permittivity
mur2 = relative permeability
sigma2 = conductivity [S/m]

==
Type of BC - always ece, bottom terminal is gnd; ev = electric voltage forced terminal; ec = electric current forced terminal
typeBC = 0,1,2,3
0 = top ev, right ev
1 = top ev, right ec
2 = top ec, right ev
3 = top ec, right ec
==
Excitation signals
imposed by the user, on non-grounded terminals; default values - first is 1 (V or A) and second is zero (V or A)
=========================== */

colorro = "LightGrey";

/* new menu for the interface - holds geometrical data */
cGUI= "{Ishape2D-ProblemData/";
mGeo = StrCat[cGUI,"Geometrical parameters/0"];
colorMGeo = "Ivory";
close_menu = 1;
DefineConstant[
  a = {2.5e-6, Name StrCat[mGeo, "0a=0.5*Dimension along Ox"], Units "m", Highlight Str[colorMGeo], Closed close_menu},
  l = {10e-6, Name StrCat[mGeo, "1l=Dimension along Oy"], Units "m", Highlight Str[colorMGeo]},
  h = {10e-6, Name StrCat[mGeo, "2h=Dimension along Oz"],Units "m", Highlight Str[colorMGeo]},
  b = {a/4, Name StrCat[mGeo, "3b=Width of the | part"],Units "m", Highlight Str[colorMGeo]},
  c = {l/3, Name StrCat[mGeo, "4c=Distance from the horizontal line of the - part"],Units "m", Highlight Str[colorMGeo]},
  d = {b, Name StrCat[mGeo, "5d=Width of the - part"],Units "m", Highlight Str[colorMGeo]}
];

/* new menu for the interface - holds material data */
mMat = StrCat[cGUI,"Material parameters/0"];
colorMMat = "AliceBlue";
DefineConstant[
  epsr1 = {1, Name StrCat[mMat, "0Relative permittivity T part"], Units "-", Highlight Str[colorMMat], Closed close_menu},
  mur1 = {1, Name StrCat[mMat, "1Relative permeability T part"], Units "-", Highlight Str[colorMMat]},
  sigma1 = {6.6e7, Name StrCat[mMat, "2Conductivity T part"], Units "S/m", Highlight Str[colorMMat]},
  epsr2 = {1, Name StrCat[mMat, "3Relative permittivity - rest"], Units "-", Highlight Str[colorMMat]},
  mur2 = {1, Name StrCat[mMat, "4Relative permeability - rest"], Units "-", Highlight Str[colorMMat]},
  sigma2 = {6.6e-7, Name StrCat[mMat, "5Conductivity - rest"], Units "S/m", Highlight Str[colorMMat]}
];

eps0 = 8.854187818e-12; // F/m - absolute permitivity of vacuum
mu0  = 4.e-7 * Pi;      // H/m - absolute permeability of vacuum
nu0  = 1/mu0;           // m/H - absolute reluctivity of vacuum

eps1 = eps0*epsr1;        // F/m - absolute permitivity
mu1 = mu0*mur1;           // F/m - absolute permeability
nu1 = 1/mu1;              // m/H - absolute reluctivity

eps2 = eps0*epsr2;        // F/m - absolute permitivity
mu2 = mu0*mur2;           // F/m - absolute permeability
nu2 = 1/mu2;              // m/H - absolute reluctivity

/* new menu for the interface - type of BC */
mTypeBC = StrCat[cGUI,"Type of BC/0"];
colorMTypeBC = "Red";
DefineConstant[
  Flag_AnalysisType = { 0,
    Choices{
      0="topTerm ev; rightTerm ev",
      1="topTerm ev; rightTerm ec",
      2="topTerm ec; rightTerm ev",
      3="topTerm ec; rightTerm ec"},
    Name Str[mTypeBC], Highlight Str[colorMTypeBC], Closed !close_menu,
    Help Str["Classical Et everywhere can be made equivalent to ECE voltage excitation",
      "Classical Et and Ht can be made equivalent to ECE current excitation",
      "This equivalences are particular for this test problem only."],
    ServerAction Str["Reset", "GetDP/1ResolutionChoices"]}
];

/* new menu for the interface - values of excitations - significance depends on the type of BC */
mValuesBC = StrCat[cGUI,"Values of BC (frequency analysis)/0"];
colorMValuesBC = "Ivory";

fmin = 1e7;   // Hz
//fmed = 3e9;   // Hz
//fmax = 100e9; // Hz

DefineConstant[
  Freq  = {fmin, Name StrCat[mValuesBC, "0Working Frequency"], Units "Hz", Highlight Str[colorMValuesBC], Closed !close_menu }
];
// You may also use UndefineConstant[]

DefineConstant[
  // Flag_AnalysisType
  // ==0 top ev, right ev
  // ==1 top ev, right ec
  // ==2 top ec, right ev
  // ==3 top ec, right ec

  VGroundRMS  = {0, Name StrCat[mValuesBC, "3Bottom/1Potential on bottom (rms)"], Units "V" , ReadOnly 1, Highlight Str[colorro],
    Visible (Flag_AnalysisType==0) || (Flag_AnalysisType==1) || (Flag_AnalysisType==2) || (Flag_AnalysisType==3)}
  VGroundPhase  = {0, Name StrCat[mValuesBC, "3Bottom/1Potential on bottom, phase"], Units "rad",  ReadOnly 1, Highlight Str[colorro],
    Visible (Flag_AnalysisType==0) || (Flag_AnalysisType==1) || (Flag_AnalysisType==2) || (Flag_AnalysisType==3)}
	
  VTerm1RMS  = {1, Name StrCat[mValuesBC, "1Top/1Potential on Term1 (rms)"], Units "V" , ReadOnly 0, Highlight Str[colorMValuesBC],
    Visible (Flag_AnalysisType==0) || (Flag_AnalysisType==1)}
  VTerm1Phase  = {0, Name StrCat[mValuesBC, "1Top/1Potential on Term1, phase"], Units "rad",  ReadOnly 0, Highlight Str[colorMValuesBC],
    Visible (Flag_AnalysisType==0) || (Flag_AnalysisType==1)}
	
  ITerm1RMS  = {1, Name StrCat[mValuesBC, "1Top/1Current through Term1 (enters the domain) (rms)"], Units "A" , ReadOnly 0, Highlight Str[colorMValuesBC],
    Visible (Flag_AnalysisType==2) || (Flag_AnalysisType==3)}
  ITerm1Phase  = {0, Name StrCat[mValuesBC, "1Top/Current through Term1 (enters the domain), phase"], Units "rad",  ReadOnly 0, Highlight Str[colorMValuesBC],
    Visible (Flag_AnalysisType==2) || (Flag_AnalysisType==3)} 	
	
  VTerm2RMS  = {0, Name StrCat[mValuesBC, "2Right/1Potential on Term2 (rms)"], Units "V" , ReadOnly 0, Highlight Str[colorMValuesBC],
    Visible (Flag_AnalysisType==0) || (Flag_AnalysisType==2)}
  VTerm2Phase  = {0, Name StrCat[mValuesBC, "2Right/1Potential on Term2, phase"], Units "rad",  ReadOnly 0, Highlight Str[colorMValuesBC],
    Visible (Flag_AnalysisType==0) || (Flag_AnalysisType==2)}	

  ITerm2RMS  = {0, Name StrCat[mValuesBC, "2Right/1Current through Term2 (enters the domain) (rms)"], Units "A" , ReadOnly 0, Highlight Str[colorMValuesBC],
    Visible (Flag_AnalysisType==1) || (Flag_AnalysisType==3)}
  ITerm2Phase  = {0, Name StrCat[mValuesBC, "2Right/Current through Term2 (enters the domain) , phase"], Units "rad",  ReadOnly 0, Highlight Str[colorMValuesBC],
    Visible (Flag_AnalysisType==1) || (Flag_AnalysisType==3)} 
];



DefineConstant[
  //_use_transfinite = {0, Choices{0,1}, Name "{MeshSettings/0Transfinite mesh?", Highlight "Yellow", Closed !close_menu}
   s = {1, Name "{MeshSettings/1Mesh factor"}
];


modelDim = 2; // 
Flag_Axi = 0; // 1 for AXI - not tested for the moment, it makes sense only for modelDim = 2

If ((modelDim == 2)&&(Flag_Axi == 0))
	h2Ddepth = h;
ElseIf ((modelDim == 2)&&(Flag_Axi == 1))   // 2D AXI
	h2Ddepth = 2*Pi;
Else // 3D
    h2Ddepth = 1;
EndIf
