/*  Ishape2d.pro
Material description (for getdp) of Ishape2D test for ECE
For details, see comments in the Ishape2d_data.pro file
The Ishape2d geometry and meshing parameters are in the Ishape2d.geo file
This file contains the setings that depend on the particular problem and
it includes 3 general files that refer to:
- computation of Jacobians
- numerical integration settings
- the weak FW formulations (where classic and ECE BC are used), and EC just to test the results for low freq.
*/

Include "Ishape2dAXI_data.pro"

Group{
  MaterialX = Region[100]; /* MaterialX - homogenous domain */
  Ground = Region[120];    /* Boundary - various parts */
  Terminal = Region[121];
  RightBoundary = Region[131];
  //LeftBoundary = Region[132];  // the axis is taken out of the domain
  
  
  Vol_FW = Region[{MaterialX}]; // domain without the boundary
  Sur_Terminals_FWece = Region[{Ground, Terminal}]; // all terminals
  //Sur_FW = Region[{Sur_Terminals_FWece, RightBoundary, LeftBoundary}]; //all boundary
  Sur_FW = Region[{Sur_Terminals_FWece, RightBoundary}]; //all boundary
  Dom_FW = Region[ {Vol_FW, Sur_FW} ];

 }

 Function{
   /* Material properties are defined piecewise, for elementary groups */
   /* the values in the right hand side are defined in Ishape2d_data.pro */

   nu[#{MaterialX,Sur_FW}] = nu;             // magnetic reluctivity [m/H]
   epsilon[#{MaterialX,Sur_FW}] = eps;       // permittivity [F/m]
   sigma[#{MaterialX,Sur_FW}]  = sigma;      // conductivity [S/m]

   // This is a problem with 2 terminals, out of which one is grounded.
   // The non-grounded can be excited in voltage or in current.
   // Depending on the excitation type, one of the following functions will be useful.
   //
   // When imposing voltage or current via a constraint, the region appears there
   // If you have a complex, the value has to be a function => use square brackets []
   VTerminal1[] =  -VBottomRMS*Complex[Cos[VBottomPhase],Sin[VBottomPhase]];
   // FW in E, ece BC, current excitation (entering the domain), time harmonic
   ITerminal1[] = -IBottomRMS*Complex[Cos[IBottomPhase],Sin[IBottomPhase]];
   
   dR[] = (Flag_Axi==0) ? Unit[ Vector[Sin[Atan2[Z[],X[]]#1], 0, Cos[#1]] ] : Vector[0,0,1] ;
 }


 Constraint{

   // ece BC
   { Name SetTerminalPotential; Type Assign;  // voltage excited terminals
     Case {
       { Region Ground; Value 0.; }
       If((Flag_AnalysisType==0))
         { Region Terminal; Value VTerminal1[]; }
       EndIf
     }
   }
   { Name SetTerminalCurrent; Type Assign;   // current excited terminals
     Case {
       If((Flag_AnalysisType==1))
         { Region Terminal; Value ITerminal1[]/h2Ddepth; }  // here the depth is needed
       EndIf
     }
   }

}

 /* The formulation and its tools */
 Include "../problemIndependent_proFiles/FullWave_E_ece_SISO.pro"

 If (Flag_AnalysisType==0)
	Dir = "res/FWeceBC_voltExc/";
 Else
	Dir = "res/FWeceBC_crtExc/";
 EndIf
 /* Output results */
Macro Change_post_options
Echo[Str[ "nv = PostProcessing.NbViews;",
    "For v In {0:nv-1}",
    "View[v].NbIso = 25;",
    "View[v].RangeType = 3;" ,// per timestep
    "View[v].ShowTime = 3;",
    "View[v].IntervalsType = 3;",
    "EndFor"
  ], File "post.opt"];
Return

Macro WriteZ_header
  Echo[ Str[Sprintf("# Hz Z RI R %g", 50)], Format Table, File > StrCat[Dir, "test_Z_RI.s1p"] ];
Return
Macro WriteY_header
  Echo[ Str[Sprintf("# Hz Y RI R %g", 50)], Format Table, File > StrCat[Dir, "test_Y_RI.s1p"] ];
Return

 PostOperation {
   { Name TransferMatrix; NameOfPostProcessing FW_E_ece ;
     Operation {
       If(Flag_AnalysisType==1)  // current excitation
         If (Freq==freqs(0))
           Call WriteZ_header;
           Printf("======================> Writting header with first Freq=%g",Freq);
         EndIf
         Print [ Vterminals, OnRegion Terminal,
           Format Table , File  > StrCat[Dir, "test_Z_RI.s1p"]] ;
         Else
         If (Freq==freqs(0))
           Call WriteY_header;
           Printf("======================> Writting header with first Freq=%g",Freq);
         EndIf
         Print [ I, OnRegion Terminal,
           Format Table , File  > StrCat[Dir, "test_Y_RI.s1p"]] ;
       EndIf
     }
   }


  { Name Maps; NameOfPostProcessing FW_E_ece ;
     Operation {
       Print [ gradV, OnElementsOf ElementsOf[Vol_FW,OnOneSideOf Sur_FW], File StrCat[Dir,"map_gradV.pos" ]];
       
       Print [ E, OnElementsOf Vol_FW, File StrCat[Dir,"map_E.pos" ]];
       Print [ J, OnElementsOf Vol_FW, File StrCat[Dir,"map_J.pos" ]];
       Print [ rmsJ, OnElementsOf Vol_FW, File StrCat[Dir,"map_absJ.pos" ]];

       // Print [ B, OnElementsOf Vol_FW, File StrCat[Dir,"map_B.pos" ]];
       Print [ H, OnElementsOf Vol_FW, File StrCat[Dir,"map_H.pos" ]];
       //Print [ Dummy, OnElementsOf Vol_FW, File StrCat[Dir,"map_dummy.pos" ]];
	   Print [ Vterminals, OnElementsOf ElementsOf[Sur_FW], File StrCat[Dir,"map_Vterminals.pos" ]];
	   Print [ VnotTerminals, OnElementsOf Vol_FW, File StrCat[Dir,"map_VnotTerminals.pos" ]];

       Call Change_post_options;
     }
  }
}
