Include "CL_data.pro";

SetFactory("OpenCASCADE");

Geometry.NumSubEdges = 100; // Visualisation

// We can activate the calculation of mesh element sizes based on curvature:
Mesh.MeshSizeFromCurvature = 1; // Needed for spiral/helix

// And we set the minimum number of elements per 2*Pi radians:
Mesh.MinimumElementsPerTwoPi = 20/2;

// We can constraint the min and max element sizes to stay within reasonnable
// values (see `t10.geo' for more details):
Mesh.MeshSizeMin = rw/4;
Mesh.MeshSizeMax = rplateC;


//==================== Winding ============================
// Using extrusion along a smooth path
vcoil()={};
pt()={};

r = rin+rw;
h = lengthInd;
npts = 10;
ptsec = npts*nturns;
end_angle = 2*Pi*nturns;

For i In {0:ptsec-1}
  thetai = i*end_angle/(ptsec-1);
  hi = i*h/(ptsec-1);
  pt()+=newp; Point(newp) = {r*Cos(thetai), r*Sin(thetai), hi};
EndFor

sp=newl; Spline(newl) = {pt()};

// Core
gap=rin/20;
vcore = news; Cylinder(news) = {0, 0, 0, 0,0,lengthInd, rin-gap};


// Connection down
// aux_pt(0) = newp; Point(newp)  = {r, -rw, -rw};
ptdw() += Rotate {{1, 0, 0}, {r, 0, -rw}, Pi/2} { Duplicata { Point{pt(0)}; } };
cen0      = newp; Point(newp)  = {r,   0, -rw};
lndw(0) = newl; Circle(newl) = {pt(0), cen0, ptdw(0)};

ptdw() += Translate {0, 0, -dCL-3*dplateC-dCL} { Duplicata { Point{ptdw(0)}; } };
lndw(1) = newl; Line(newl)={ptdw()};


// Connection up
ptup() += Rotate {{1, 0, 0}, {r*Cos(thetai), r*Sin(thetai), hi+rw}, Pi/2} { Duplicata { Point{pt(#pt()-1)}; } };
cen1 = newp; Point(newp) = {r*Cos(thetai), r*Sin(thetai), hi+rw};
lnup(0) = newl; Circle(newl) = {pt(#pt()-1), cen1, ptup()};

ptup() += Translate {0, 0, dCL} { Duplicata { Point{ptup(0)}; } };
lnup(1) = newl; Line(newl)={ptup()};


// A wire is like a curve loop, but open:
ww=newl; Wire(newl) = {lndw(1), lndw(0), sp, lnup()};

// We define the shape we would like to extrude along the spline (a disk):
coor_cen_disk() = Point{ptdw(1)};
ss = news; Disk(news) = {coor_cen_disk(), rw};

// We extrude the disk along the spline to create a pipe:
vcoil() += Extrude { Surface{ss}; } Using Wire {ww};

// We delete the source surface, and increase the number of sub-edges for a
// nicer display of the geometry:
Delete{ Point{pt()};}
Recursive Delete{ Line{lndw(1), lndw(0), sp, lnup()}; }
Recursive Delete{ Surface{ss}; }

co_plate() = coor_cen_disk();
co_plate(2)= coor_cen_disk(2)+3*dplateC+dCL;

vplate(0) = news; Cylinder(news) = {co_plate(), 0, 0, -dplateC, rplateC};
vplate(1) = news; Cylinder(news) = {co_plate({0,1}),co_plate(2)-dplateC,   0, 0, -dplateC, rplateC};
vplate(2) = news; Cylinder(news) = {co_plate({0,1}),co_plate(2)-2*dplateC, 0, 0, -dplateC, rplateC};

vol()=BooleanFragments{Volume{Volume{:}};Delete;}{};

bb() = BoundingBox Volume {vol()};
// Printf("",bb());

sizeX = bb(3)-bb(0);
sizeY = bb(4)-bb(1);
sizeZ = bb(5)-bb(2);
// Printf("sizeZ",sizeZ);

// Translate {bb(0)+sizeX/2, bb(1)+sizeY/2, bb(2)+sizeZ/2} {
//  Volume{Volume{:}}; Surface{Surface{:}}; Line{Line{:}}; Point{Point{:}};}


//----------------------------------------------------------------
//------------------- AIR ----------------------------------------
//----------------------------------------------------------------
xb = sizeX*1.5;
yb = sizeY*1.5;
zb = sizeZ*0.995;
z0 = co_plate(2)-3*dplateC-dCL;

// Create a box
p = newp;
Point(p)   = {xb, -yb, z0};
Point(p+1) = {xb,  yb, z0};
Point(p+2) = {-xb, yb, z0};
Point(p+3) = {-xb,-yb, z0};

Point(p+4) = {xb, -yb, z0+zb};
Point(p+5) = {xb,  yb, z0+zb};
Point(p+6) = {-xb, yb, z0+zb};
Point(p+7) = {-xb,-yb, z0+zb};

l = newl;
Line(l) = {p,p+1};
Line(l+1) = {p+1,p+2};
Line(l+2) = {p+2,p+3};
Line(l+3) = {p+3,p};
Line(l+4) = {p+4,p+5};
Line(l+5) = {p+5,p+6};
Line(l+6) = {p+6,p+7};
Line(l+7) = {p+7,p+4};
Line(l+8) = {p, p+4};
Line(l+9) = {p+1, p+5};
Line(l+10) = {p+2, p+6};
Line(l+11) = {p+3, p+7};

llb = newll;
Line Loop(llb)   = {l+8, -(l+7), -(l+11), l+3};
Line Loop(llb+1) = {l+9, l+5, -(l+10), -(l+1)};
Line Loop(llb+2) = {l,l+1,l+2,l+3};
Line Loop(llb+3) = {l+4,l+5,l+6,l+7};
Line Loop(llb+4) = {l+2, l+11, -(l+6), -(l+10)};
Line Loop(llb+5) = {l, l+9, -(l+4), -(l+8)};


surf_box() += news; Plane Surface(news) = {llb};
surf_box() += news; Plane Surface(news) = {llb+1};
surf_box() += news; Plane Surface(news) = {llb+2};
surf_box() += news; Plane Surface(news) = {llb+3};
surf_box() += news; Plane Surface(news) = {llb+4};
surf_box() += news; Plane Surface(news) = {llb+5};
sl_box = newsl; Surface Loop(newsl) = surf_box();
vair_aux = newv; Volume(newv) = {sl_box};

all_vol()=BooleanFragments{Volume{Volume{:}};Delete;}{};
// Printf("all ",all_vol());
Recursive Delete {Volume{all_vol(8)};}

vol_ind() = all_vol(9);
vol_wire() = all_vol(4);
vol_plate() = all_vol({1,3,5,7});

vol_cond() = {vol_ind(),vol_wire(),vol_plate()};
vol_diel() = all_vol({2,6});
vol_air = all_vol(#all_vol()-1);

Characteristic Length {PointsOf{Volume{vol_plate()};}} = rplateC/8/2; // Plates and dielectric
Characteristic Length {PointsOf{Volume{vol_diel()};}}  = rplateC/8/2; // /32; // Plates and dielectric
Characteristic Length {PointsOf{Volume{vcore};}} = rin/8;


// Improving mesh around plates...
vmesh = news; Cylinder(news) = {co_plate(), 0, 0, -3*dplateC, 1.1*rplateC};
bnd_vmesh() = Abs(Boundary{Volume{vmesh};});
// Printf("",bnd_vmesh());

Delete{ Volume{vmesh}; Surface{bnd_vmesh({1,2})};}
Surface{bnd_vmesh(0)} In Volume {vol_air};
Characteristic Length {PointsOf{Surface{bnd_vmesh(0)};}}  = rplateC/8/2;
Recursive Hide {Surface {bnd_vmesh(0)};}


// Physical regions
// --------------------------
//Physical Volume("Air", AIR) = vol_air;
Physical Volume("DomainAir", AIR) = vol_air;

//Physical Volume("Conductor IN  (plate + ind)",  CONDUCTOR) = {vol_ind(), vol_plate({0,2})};
//Physical Volume("Conductor OUT (plate + wire)", CONDUCTOR+1) = {vol_plate({1,3}), vol_wire()};
Physical Volume("DomainConductor",  CONDUCTOR) = {vol_ind(), vol_plate({0,2}),vol_plate({1,3}), vol_wire()};

//Physical Volume("Dielectric", DIELECTRIC) = vol_diel();
Physical Volume("DomainDielectric", DIELECTRIC) = vol_diel();

//Physical Volume("Core", CORE) = vcore;
Physical Volume("DomainCore", CORE) = vcore;


surf_elec_in =28;
surf_elec_out=18;
//Physical Surface("Electrode IN",  ELEC_IN)  = surf_elec_in;
Physical Surface("Terminal",  ELEC_IN)  = surf_elec_in;
//Physical Surface("Electrode OUT", ELEC_OUT) = surf_elec_out;
Physical Surface("Ground", ELEC_OUT) = surf_elec_out;
Physical Surface("Plate IN",  ELEC_DIEL_IN)  = {23,14};
Physical Surface("Plate OUT", ELEC_DIEL_OUT) = {20,12};

bnd_cond_in() = Abs(CombinedBoundary{Volume{vol_ind(), vol_plate({0,2})};});
bnd_cond_in() -= {surf_elec_in};
Physical Surface("Boundary conductor IN", BND_CONDUCTOR) = bnd_cond_in();

bnd_cond_out() = Abs(CombinedBoundary{Volume{vol_plate({1,3}), vol_wire()};});
bnd_cond_out() -= {surf_elec_out};
Physical Surface("Boundary conductor OUT", BND_CONDUCTOR+1) = bnd_cond_out();

bnd_cond() = Abs(CombinedBoundary{Volume{vol_cond(), vol_diel()};});
// bnd_cond() = Abs(CombinedBoundary{Volume{vol_cond()};});
bnd_cond() -= {surf_elec_in,surf_elec_out};
Physical Surface("Boundary conductor", BND_CONDUCTOR+2) = bnd_cond();

bnd_air() = Abs(CombinedBoundary{Volume{Volume{:}};});
bnd_air() -= {surf_elec_in,surf_elec_out};
//Physical Surface("Outer boundary", BND_AIR) = bnd_air();
Physical Surface("SurBCnotTerminalAir", BND_AIR) = bnd_air();


// For aestetics
Recursive Color SkyBlue { Volume{vol_air()};}
Recursive Color Green  { Volume{vcore};}
Recursive Color Red  { Volume{vol_cond()};}
Recursive Color Yellow  { Volume{vol_diel()};}
