function [ok] = savecirDEM(filename,N,qr,qc,realpl,cplxpl,...
    kinf,kreal,kcplx,kzero,...
    ptype,fmin,fmax,anscir)

% savecirDEM = circuit synthesis using 
%              Differential Equation Macromodel method 
%              (the input is the transfer matrix given
%              as poles and redidues, and the output is a Spice netlist)
% Reference: Timo Palenius and Janne Roos, "Comparison of Reduced-Order
%            Interconnect Macromodels for Time-Domain Simulation",
%            IEEE Transaction on Microwave Theory and Techniques,
%            Vol.52, No.9, pp.2240-2250,2004.
% Parameters:
%   filename - The name of the output SPICE file. The reccomended
%              extension you should use is *.cir; If its extension is not
%              .cir, than .cir will be added to the end of the name
%              (string of characters)
%   N  - The number of terminals (integer)
%   qr - The number or real poles (integer)
%   qc - The number of pairs of complex poles 
%        (the total no of poles is qr + 2 qc)  (integer)
%   realpl - real poles - real vector of length qr
%   cplxpl - complex poles - complex vector of length qc
%   kinf - coeficient matrix that links directly the output to the input
%         (kinf is zero if the transfer function is strictly proper and 
%          non zero otherwise) 
%   kreal - array of matrices, the residuals corresponding to the real
%           poles
%   kcplx - array of matrices, the residuals corresponding to the complex
%           poles
%   kzero - coef that multiplies s in the transfer function; it is not zero
%           only for improper functions
%   ptype - character; it can be 'Y' or 'Z'. Usefull to know how the
%           excitation of a main circuit must be so that the answer have
%           a certain physical meaning (current or voltage, respectively)
%   fmin - min frequency - to be written in the excitation type of the
%          terminals
%   fmax - max frequency - to be written in the excitation type of the
%          terminals
%   anscir - string that can be 'yes' or 'no'. If it is not no, an example
%          of main circuit will be added to the file, otherwise, the file
%          will contain only the subcircuit. Use 'yes' if you want to
%          easily test the file in SPICE
%   ok - output flag; ok = 0 means normal (succesfull) top
% ============================================================
% Code developed with partial financial support from projects:
% FP5/IST/CODESTAR (http://www.imec.be/codestar)

% Designer(s) of algorithms and data structures: 
% Gabriela Ciuprina, Daniel Ioan
% Author of source code: Gabriela Ciuprina
% Tested by: Gabriela Ciuprina

% Copyright LMN (2006), http://www.lmn.pub.ro

% Name of file: savecirDEM.m
% Version: 1.0
% Last update: 15 October 2007

% Note: this function is included in the RomWorkBench Tool
% developed in the frame of the FP5/IST/CODESTAR project
% (see deliverable D15_LMN for details)
%=======================


ok = 0;
Nfisiere = N;
if strcmp(anscir,'no');
    Nfisiere = 1;
end

for noterm = 1:Nfisiere
    if noterm == 1
        ll = length(filename);
        ext = filename(ll-4:ll);
        if strcmp(ext,'.cir')
            filenamesave = filename(1:ll-4);
        else
            filenamesave = filename;
            ext = '.cir';
        end
    else
        filename = strcat(filenamesave,'_Term',num2str(noterm),ext);
        fprintf('cir file no %d is %s \n',noterm,filename);
    end
    fid=fopen(filename,'w');
    fprintf(fid,...
        '*****************************************************************************\n');
    fprintf(fid,...
        '* SPICE equivalent circuit generated by Codestar snp2cir developed in UPB/LMN\n');
    fprintf(fid,...
        '*****************************************************************************\n');
    fprintf(fid,'*Differential-equation macromodel\n');
    fprintf(fid,'\n \n');

    fprintf(fid,'.SUBCKT SYNT');
    for poarta = 1:N
        fprintf(fid,' Term%d ', poarta);
    end
    fprintf(fid,'  OUT');
    fprintf(fid,'\n \n');

    ptype = upper(ptype);
    tipinput = zeros(1,N);
    for poarta = 1:N
        if strcmp(ptype,'Y') == 1
            tipinput(poarta)='V';
        elseif strcmp(ptype,'Z') == 1
            tipinput(poarta)='I';
        else
            disp('savecirDEM.m: ptype must be Z or Y.');
            disp('Export SPICE file (subcircuit and main circuit) - FAILURE');
            ok = 1;
            return;
        end;
    end;


    for poarta = 1:N
        if strcmp(char(tipinput(poarta)),'V')
            for j = 1:N
                if kinf(poarta,j) ~= 0
                    if strcmp(char(tipinput(j)),'V')
                        fprintf(fid,'Ginf%d_%d Term%d OUT Term%d OUT %e\n', ...
                            poarta,j,poarta,j,kinf(poarta,j)); %SICU
                    else
                        fprintf(fid,'Finf%d_%d Term%d OUT Vprobe%d %e\n', ...
                            poarta,j,poarta,j,kinf(poarta,j)); %SICI
                    end
                end
                if kzero(poarta,j) ~= 0
                    if poarta == j
                        fprintf(fid,'Cprop%d Term%d middle%d %e\n', ...
                            poarta,poarta,poarta,kzero(poarta,j));
                        fprintf(fid,'Vmiddle%d middle%d OUT 0\n',poarta,poarta);
                    else
                        if kzero(j,j) ~= 0
                            if strcmp(char(tipinput(j)),'V')
                                fprintf(fid,'Fprop%d_%d Term%d OUT Vmiddle%d %e\n', ...
                                    poarta,j,poarta,j,kzero(poarta,j)/kzero(j,j)); %SICI
                            else
                                fprintf(fid,'Gprop%d_%d Term%d OUT new%d_1 new%d_2  %e\n', ...
                                    poarta,j,poarta,j,j,kzero(poarta,j)/kzero(j,j)); %SICU
                            end
                        end

                    end
                end
                for m = 1:qr
                    if kreal(poarta,j,m) ~= 0
                        fprintf(fid,'Greal%d_%d_%d Term%d OUT nod%d_%d 0 %e\n',...
                            poarta,j,m,poarta,j,m,kreal(poarta,j,m)); %SICU
                    end
                end
                for m=1:qc
                    rr = real(kcplx(poarta,j,m));
                    if rr ~= 0
                        fprintf(fid,'GcplxR%d_%d_%d Term%d OUT left%d_%d 0 %e\n',...
                            poarta,j,m,poarta,j,m,-sqrt(2)*rr); %SICU
                    end
                    im = imag(kcplx(poarta,j,m));
                    if im ~= 0
                        fprintf(fid,'GcplxI%d_%d_%d Term%d OUT right%d_%d 0 %e\n',...
                            poarta,j,m,poarta,j,m,sqrt(2)*im); %SICU
                    end;
                end;
            end;

        else % poarta are intrarea in curent si iesirea in tensiune
            index = 1;
            fprintf(fid,'Vprobe%d Term%d new%d_%d 0 \n',poarta,poarta,poarta,index);
            fprintf(fid,'\n \n');
            for j = 1:N
                if kzero(poarta,j) ~= 0
                    if poarta == j
                        fprintf(fid,'Lprop%d new%d_%d new%d_%d %e \n',...
                            poarta,poarta,index,poarta,index+1,kzero(poarta,j));
                        index = index + 1;
                    else
                        if kzero(j,j) ~= 0
                            if strcmp(char(tipinput(j)),'V')
                                fprintf(fid,'Hmiddle%d_%d new%d_%d new%d_%d Vmiddle%d %e\n',...
                                    poarta,j,poarta,index,poarta,index+1,j,kzero(poarta,j)/kzero(j,j)); %SUCI
                                index = index + 1;
                            else
                                fprintf(fid,'Emiddle%d_%d new%d_%d new%d_%d new%d_1 new%d_2 %e\n',...
                                    poarta,j,poarta,index,poarta,index+1,j,j,kzero(poarta,j)/kzero(j,j));  %SUCU
                            end
                        end
                    end
                end
                if kinf(poarta,j) ~= 0
                    if strcmp(char(tipinput(j)),'V')
                        fprintf(fid,'Einf%d_%d new%d_%d new%d_%d Term%d OUT %e\n',...
                            poarta,j,poarta,index,poarta,index+1,j,kinf(poarta,j));  %SUCU
                        index = index + 1;
                    else
                        fprintf(fid,'Hinf%d_%d new%d_%d new%d_%d Vprobe%d %e\n',...
                            poarta,j,poarta,index,poarta,index+1,j,kinf(poarta,j)); %SUCI
                        index = index + 1;
                    end
                end
                for m = 1:qr
                    if kreal(poarta,j,m) ~= 0
                        fprintf(fid,'Ereal%d_%d_%d new%d_%d new%d_%d nod%d_%d 0 %e\n',...
                            poarta,j,m,poarta,index,poarta,index+1,j,m,kreal(poarta,j,m)); %SUCI
                        index = index + 1;
                    end

                end
                for m=1:qc
                    rr = real(kcplx(poarta,j,m));
                    if rr ~= 0
                        fprintf(fid,'EcplxR%d_%d_%d new%d_%d new%d_%d left%d_%d 0 %e\n',...
                            poarta,j,m,poarta,index,poarta,index+1,j,m,-sqrt(2)*rr);%SUCI
                        index=index+1;
                    end
                    im = imag(kcplx(poarta,j,m));
                    if im ~= 0
                        fprintf(fid,'EcplxI%d_%d_%d new%d_%d new%d_%d right%d_%d  0 %e\n',...
                            poarta,j,m,poarta,index,poarta,index+1,j,m,sqrt(2)*im); %SUCI
                        index = index + 1;
                    end
                end;
            end;
            fprintf(fid,'Vbis%d new%d_%d OUT 0\n',poarta,poarta,index);

        end;
    end;

    fprintf(fid,'\n \n');
    fprintf(fid,'*elements corresponding to the real poles: \n');
    fprintf(fid,'*========================================: \n');

    for i = 1:N
        for j = 1:qr
            fprintf(fid,'\n');
            fprintf(fid,'*Real pole circuit %d %d\n',i,j);
            if strcmp(char(tipinput(i)),'V')
                fprintf(fid,'Ga%d_%d 0 nod%d_%d Term%d OUT 1\n',i,j,i,j,i); % SICU
            else
                fprintf(fid,'Fa%d_%d 0 nod%d_%d Vprobe%d 1\n',i,j,i,j,i); % SICI
            end
            fprintf(fid,'Ca%d_%d nod%d_%d 0 1\n',i,j,i,j);
            if realpl(j) ~= 0
                rez = -1/realpl(j);
            else
                rez = Inf;
            end
            if rez ~= Inf
                fprintf(fid,'Ra%d_%d nod%d_%d 0 %e\n',i,j,i,j,rez);
            end;
        end;
    end;

    fprintf(fid,'\n \n');
    fprintf(fid,'*elements corresponding to the complex poles: \n');
    fprintf(fid,'*========================================: \n');

    for i = 1:N
        for j = 1:qc
            % left
            fprintf(fid,'\n');
            fprintf(fid,'*Complex pole circuit %d %d\n',i,j);
            if strcmp(char(tipinput(i)),'V')
                fprintf(fid,'Gb%d_%d_first 0 left%d_%d Term%d OUT %e\n',...
                    i,j,i,j,i,-sqrt(2)); %SICU
            else
                fprintf(fid,'Fb%d_%d_first 0 left%d_%d Vprobe%d %e\n',...
                    i,j,i,j,i,-sqrt(2)); %SICI
            end
            fprintf(fid,'Cb%d_%d_first left%d_%d 0 1 \n',i,j,i,j);
            if real(cplxpl(j)) ~= 0
                rez = -1/real(cplxpl(j));
            else
                rez = Inf;
            end
            if rez ~= Inf
                fprintf(fid,'Rb%d_%d_first left%d_%d 0 %e \n',i,j,i,j,rez);
            end;
            im = imag(cplxpl(j));
            if im ~= 0
                %if strcmp(char(tipinput(i)),'V')
                fprintf(fid,'Gb%d_%d_second left%d_%d 0 right%d_%d 0 %e \n',i,j,i,j,i,j,im); %SICU
                % right
                fprintf(fid,'Gb%d_%d_third 0 right%d_%d left%d_%d 0 %e \n',i,j,i,j,i,j,im); %SICU
                %else
                %fprintf(fid,'Fb%d_%d_second left%d_%d 0 Vprobe%d %e \n',i,j,i,j,i,im); %SICI
                % right
                %fprintf(fid,'Fb%d_%d_third 0 right%d_%d Vprobe%d %e \n',i,j,i,j,i,im); %SICI
                %end
            end
            fprintf(fid,'Cb%d_%d_second right%d_%d 0 1\n',i,j,i,j);
            if rez ~= Inf
                fprintf(fid,'Rb%d_%d_second right%d_%d 0 %e\n',i,j,i,j,rez);
            end;
        end;
    end;
    fprintf(fid,'\n');
    fprintf(fid,'.ENDS SYNT');

    if ~strcmpi(anscir,'NO')
        answer = cell(1,N);
        %disp(strcat('Input ',' ',num2str(N),' SPICE like excitations for the terminals (default is AC 1):'));
        for i = 1:N
            %ans = input( strcat('Source ',num2str(i),'    ',num2str(poarta),' ? '));
            %if isempty(ans)
            answer{i} = 'AC 0';
            %else
            %    answer{i} = char(ans);
            %end
        end
        answer{noterm} = 'AC 1';


        fprintf(fid,'\n \n');
        fprintf(fid,'************************* \n');
        fprintf(fid,'*Example of Main circuit: \n');
        fprintf(fid,'************************* \n');
        for poarta=1:N
            if strcmp(char(tipinput(poarta)),'V')
                fprintf(fid,'V%d Term%d %d %s \n',poarta, poarta, 0, answer{poarta});
            else
                fprintf(fid,'I%d %d Term%d %s \n',poarta, 0, poarta, answer{poarta});
            end
        end
        fprintf(fid,'\n');
        fprintf(fid,'X1');
        for poarta = 1:N
            fprintf(fid,' Term%d ', poarta);
        end
        fprintf(fid,'  0');
        fprintf(fid,' SYNT\n \n');

        fprintf(fid,'.PROBE\n');

        %answer = input('Input SPICE simulation type (default is .AC DEC 100 1e8 1e11)');
        %if isempty(answer)
        answer     = {'.AC LIN 100 '};
        %else
        %    answer = char(answer);
        %end
        fprintf(fid,answer{1});
        fprintf(fid,' %e', fmin);
        fprintf(fid,' %e', fmax);
        fprintf(fid,'\n');



        fprintf(fid,'.END\n');
        fprintf(fid,'\n');
    end
    fclose(fid);

end
disp('Export SPICE file (subcircuit and main circuit) - DONE');

